"""CLI handling utilities"""

import os
import argparse
from . import NotionExporter
from .utils import pages_from_str


def parse_args() -> argparse.Namespace:
    """Parses the CLI arguments.
    Only `page_id` is required. All other arguments are optional.
    """
    parser = argparse.ArgumentParser(
        description="Export Notion pages to your GitHub repository."
    )
    parser.add_argument(
        "page_id",
        help="The ID of the Notion page you want to export. Separate multiple pages with commas.",
    )
    parser.add_argument(
        "-d",
        "--export_directory",
        help="The directory to export the page to. Defaults to the current directory.",
    )
    parser.add_argument(
        "-k",
        "--keep_structure",
        help="Keep the page hierarchy when exporting. Defaults to False.",
        action="store_true",
    )
    parser.add_argument(
        "-t",
        "--export_type",
        help="The type of file to export. Defaults to HTML.",
        choices=["html", "markdown", "pdf", "word"],
        default="html",
    )
    parser.add_argument(
        "-c",
        "--current_view_export_type",
        help="The type of view to export. Defaults to the currentView.",
        choices=["currentView", "all"],
        default="currentView",
    )
    parser.add_argument(
        "-l",
        "--no-file",
        help="Do not export files attached to the page. Defaults to False.",
        action="store_true",
    )
    parser.add_argument(
        "-s",
        "--single_page",
        help="Do not export child pages. Defaults to False.",
        action="store_true",
    )
    parser.add_argument(
        "-w",
        "--rewrite",
        help="Do not create date-formated folders for each export. Defaults to False.",
        action="store_true",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Whether to print verbose output. Defaults to False.",
        action="store_true",
    )
    args = parser.parse_args()
    return args


def run_from_cli():
    """Runs the exporter from the CLI arguments."""
    args = parse_args()
    token_v2 = os.getenv("NOTION_TOKEN_V2")
    file_token = os.getenv("NOTION_FILE_TOKEN")
    if not token_v2:
        raise ValueError("NOTION_TOKEN_V2 environment variable not set.")
    if not file_token:
        raise ValueError("NOTION_FILE_TOKEN environment variable not set.")
    exporter = NotionExporter(
        token_v2=token_v2,
        file_token=file_token,
        pages=pages_from_str(args.page_id),
        export_directory=args.export_directory,
        flatten_export_file_tree=args.keep_structure,
        export_type=args.export_type,
        current_view_export_type=args.current_view_export_type,
        include_files=not args.no_file,
        recursive=not args.single_page,
        rewrite=args.rewrite,
        verbose=args.verbose,
    )
    exporter.export()



import json
import logging
import os
import time
from datetime import datetime
from concurrent.futures import ThreadPoolExecutor
from typing import Tuple, Dict
from tqdm import tqdm

from . import utils


logger = logging.getLogger(__name__)


class ExportType:
    MARKDOWN = "markdown"
    HTML = "html"
    PDF = "pdf"


class ViewExportType:
    CURRENT_VIEW = "currentView"
    ALL = "all"


class NotionExporter:
    """
    Uses the Notion API to export pages asynchronously. Downloads the exported zip file
    and unpacks it to the specified directory.

    ## Arguments
    - `token_v2` The token_v2 cookie from your logged-in Notion session.
    - `file_token` The file_token cookie from your logged-in Notion session.
    - `pages` A {'name' : 'id'} format dict. **Names can be anything you want**.
    - `export_directory` The directory to export the pages to. Defaults to the current directory.
    - `flatten_export_file_tree` Whether to flatten the export file tree. Defaults to True.
    - `export_type` The type of export to perform. Defaults to `ExportType.MARKDOWN`.
    - `current_view_export_type` The type of export to perform. Defaults to `ViewExportType.CURRENT_VIEW`.
    - `include_files` Whether to include the static files in the export. Defaults to True.
    - `recursive` Whether to recursively export subpages. Defaults to True.
    - `rewrite` Whether to rewrite the export directory. Defaults to False. If True, does not create a new directory for each export.
    - `verbose` Whether to print verbose output. Defaults to False.
    """

    def __init__(
        self,
        token_v2: str,
        file_token: str,
        pages: Dict[str, str],
        export_directory: str = None,
        flatten_export_file_tree: bool = True,
        export_type: ExportType = ExportType.MARKDOWN,
        current_view_export_type: ViewExportType = ViewExportType.CURRENT_VIEW,
        include_files: bool = True,
        recursive: bool = True,
        rewrite: bool = False,
        verbose: bool = False,
    ):
        for v in pages.values():
            utils.to_uuid_format(v, error_msg=f"Invalid page ID. ")
        self.token_v2 = token_v2
        self.file_token = file_token
        self.include_files = include_files
        self.recursive = recursive
        self.pages = pages
        self.current_view_export_type = current_view_export_type
        self.flatten_export_file_tree = flatten_export_file_tree
        self.export_type = export_type
        self.download_headers = {
            "content-type": "application/json",
            "cookie": f"file_token={self.file_token};",
        }
        self.query_headers = {
            "content-type": "application/json",
            "cookie": f"token_v2={self.token_v2};",
        }
        self.export_directory = export_directory or os.getcwd()
        if not self.export_directory.startswith("/"):
            self.export_directory = f"{os.getcwd()}/{self.export_directory}"
        dir = datetime.now().strftime("%Y-%m-%d_%H-%M-%S") if not rewrite else ""
        os.makedirs(f"{self.export_directory}/{dir}", exist_ok=True)
        self.export_directory = f"{self.export_directory}/{dir}"
        if verbose:
            logging.basicConfig(
                level=logging.DEBUG,
                format="%(asctime)s [%(levelname)s] [%(module)s.%(funcName)s]: %(message)s",
                datefmt="%Y-%m-%d %H:%M:%S",
            )

    def _get_format_options(
        self, export_type: ExportType, include_files: bool = False
    ) -> dict:
        format_options = {}
        if export_type == ExportType.PDF:
            format_options["pdfFormat"] = "Letter"

        if not include_files:
            format_options["includeContents"] = "no_files"

        return format_options

    def _get_task_id(self, id: str) -> str:
        url = "https://www.notion.so/api/v3/enqueueTask"
        id = utils.to_uuid_format(id)
        export_options = {
            "exportType": self.export_type,
            "locale": "en",
            "timeZone": "Europe/London",
            "collectionViewExportType": self.current_view_export_type,
            "flattenExportFiletree": self.flatten_export_file_tree,
        }
        # Update the exportOptions with format-specific options
        export_options.update(
            self._get_format_options(
                export_type=self.export_type, include_files=self.include_files
            )
        )
        payload = json.dumps(
            {
                "task": {
                    "eventName": "exportBlock",
                    "request": {
                        "block": {
                            "id": id,
                        },
                        "recursive": self.recursive,
                        "exportOptions": export_options,
                    },
                }
            }
        )
        response = utils.session.post(url, headers=self.query_headers, data=payload)
        response.raise_for_status()
        return response.json()["taskId"]

    def _get_status(self, task_id: str) -> dict:
        url = "https://www.notion.so/api/v3/getTasks"

        payload = json.dumps({"taskIds": [task_id]})

        response = utils.session.post(url, headers=self.query_headers, data=payload)
        response.raise_for_status()
        return response.json()["results"][0]

    def _process_page(self, page: Tuple[str, str]) -> dict:
        name, id = page
        task_id = self._get_task_id(id)

        status, state, error, pages_exported = self._wait_for_export_completion(
            task_id=task_id
        )
        if state == "failure":
            logger.error(f"Export failed for {name} with error: {error}")
            return {"state": state, "name": name, "error": error}

        export_url = status.get("status", {}).get("exportURL")
        if export_url:
            utils.download_url(
                url=export_url,
                path=f"{self.export_directory}/{export_url.split('/')[-1]}",
                headers=self.download_headers,
            )
        else:
            logger.error(f"Failed to get exportURL for {name}")

        return {
            "state": state,
            "name": name,
            "exportURL": export_url,
            "pagesExported": pages_exported,
        }

    def _wait_for_export_completion(self, task_id) -> Tuple[dict, str, str, int]:
        """Helper method to wait until the export is complete or failed."""
        while True:
            status = self._get_status(task_id)
            if not status:
                time.sleep(1)
                continue
            state = status.get("state")
            error = status.get("error")
            if state == "failure" or status.get("status", {}).get("exportURL"):
                return (
                    status,
                    state,
                    error,
                    status.get("status", {}).get("pagesExported"),
                )
            time.sleep(1)

    def export(self):
        logger.info(f"Exporting {len(self.pages)} pages...")
        with ThreadPoolExecutor() as executor:
            futures = []
            with tqdm(total=len(self.pages), dynamic_ncols=True) as pbar:
                for page in self.pages.items():
                    futures.append(executor.submit(self._process_page, page))
                for future in futures:
                    result = future.result()
                    if result["state"] == "failure":
                        continue
                    name = result["name"]
                    pagesExported = result["pagesExported"]

                    pbar.set_postfix_str(
                        f"Exporting {name}... {pagesExported} pages already exported"
                    )
                    pbar.update(1)

        utils.unzip_all(self.export_directory)
        # Rename the extracted pages to the page names provided from self.pages
        utils.rename_extracted_pages(
            path=self.export_directory, pages=self.pages, format=self.export_type
        )



"""Utilities for notiondl."""

import os
import shutil
import requests
import logging

session = requests.Session()
logger = logging.getLogger(__name__)


def unzip_all(path: str) -> None:
    """Unzips all the packed files in a directory."""
    logger.info(f"Unzipping all files in {path}")
    for file in os.listdir(path):
        if file.endswith(".zip"):
            full_file_path = os.path.join(path, file)
            shutil.unpack_archive(full_file_path, path, "zip")
            os.remove(full_file_path)
    logger.info(f"Unzipped all files in {path}")


def shorten_file_name(path: str, n: int = 255) -> str:
    """Shortens a file name to `n` characters. Defaults to 255."""
    # pick the first part if there is '?' in the path
    path = path.split("?")[0]
    if len(path.split("/")[-1]) < n + 1:
        return path
    part = path.split("/")[-1]
    # pick the last 255 characters if still too long
    cut = part[len(part) - n :]
    if len(part) > n:
        path = "/".join(path.split("/")[:-1]) + "/" + cut
    return path


def download_url(url: str, path: str, **kwargs) -> None:
    """Downloads a file from a url to a path."""
    logger.info(f"Downloading {url} to {path}")
    p = shorten_file_name(path)
    response = session.get(url, stream=True, **kwargs)
    response.raise_for_status()
    with open(p, "wb") as f:
        shutil.copyfileobj(response.raw, f)


def rename_extracted_pages(path: str, pages: dict, format: str) -> None:
    """Renames the extracted pages to their original names."""
    for file in os.listdir(path):
        if file.endswith(format):
            fname = file.split(".")[0]
            for k, v in pages.items():
                if fname.split()[-1] == v:
                    new = k if k.endswith(format) else k + "." + format
                    os.rename(f"{path}/{file}", f"{path}/{new}")


def pages_from_str(page_id: str) -> dict:
    """Constructs the pages dict from a comma separated string.
    <NAME>:<ID> format is supported."""
    pages = {}
    for page_id in page_id.split(","):
        if ":" in page_id:
            if len(page_id.split(":")) != 2:
                raise ValueError(
                    f"Invalid page ID format: {page_id}. Use <NAME>:<ID> format."
                )
            name, id = page_id.split(":")
            pages[name] = id
        else:
            pages[page_id] = page_id
    return pages


def to_uuid_format(s: str, error_msg: str = None) -> str:
    """Converts a string to the uuid format."""
    if len(s) != 32:
        raise ValueError(f"{error_msg}A UUID must be 32 characters long. given length: {len(s)}")
    if "-" == s[8] and "-" == s[13] and "-" == s[18] and "-" == s[23]:
        return s
    return f"{s[:8]}-{s[8:12]}-{s[12:16]}-{s[16:20]}-{s[20:]}"
